/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
 
/*
 *
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.monitorapij.ant;

import java.io.File;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import org.apache.axis.utils.Admin;
import org.apache.axis.utils.XMLUtils;
import org.apache.axis.AxisEngine;
import org.apache.axis.server.AxisServer;
import org.apache.axis.deployment.wsdd.WSDDDocument;
import org.apache.axis.WSDDEngineConfiguration;
import org.apache.axis.deployment.wsdd.WSDDDeployment;
import org.w3c.dom.Document;

public class ServerConfigParser extends DefaultHandler {

    private Writer writer;
    private String serviceName;
    private String version;
    private boolean tagIncomplete;
    private int indent;

    private String currentService;
    private String currentParamName;
    private String currentTag;
    private String currentToken;
    

    public ServerConfigParser(Writer writer, String serviceName, String version){

        this.writer = writer;
        this.serviceName = serviceName;
        this.version = version;
        indent = 0;
        currentService = "";
        currentParamName = "";
        currentTag = "";
        currentToken = null;

    }

    public void startElement(String uri, String name, String qName, Attributes attributes)
        throws SAXException {

        try{

            if( tagIncomplete ){
                writer.write(">\n");

                if( currentTag.equals("service") &&
                    currentService.equals(serviceName) ){
                    indent();
                    writer.write("<requestFlow>\n");
                    indent();
                    writer.write(" <handler type=\"java:org.glite.ce.commonj.axis.handlers.AuthorizationHandler\"/>\n");
                    indent();
                    writer.write("</requestFlow>\n");
                    indent();
                    writer.write("<parameter name=\"monitorVersion\" value=\"");
                    writer.write(version);
                    writer.write("\"/>\n");
                    indent();
                    writer.write("<parameter name=\"monitorDescription\" value=\"CE-Monitor_web_application\"/>\n");

                }
            }


            if( qName.equalsIgnoreCase("service") ){
                currentService = attributes.getValue("name");
            }else if( qName.equalsIgnoreCase("parameter") ){
                currentParamName = attributes.getValue("name");
            }

            currentTag = qName;
            currentToken = null;

            indent();
            writer.write("<" + qName);
            for(int k=0; k< attributes.getLength(); k++){
            
                String attrQName = attributes.getQName(k);
                String attrValue = attributes.getValue(k);

                writer.write(" " + attrQName + "=\"");
                if( !currentService.equals(serviceName) ){

                    writer.write(attrValue + "\"");

                }else if( attrQName.equals("qname") && attrValue.indexOf(":>")>0 ){

                    writer.write(attrValue.replaceFirst(":>",":&gt;") + "\"");

                }else if( attrQName.equals("returnQName") &&
                          qName.equalsIgnoreCase("operation") ){

                    String tmpAttr = attributes.getValue("returnType");
                    if( tmpAttr!=null ){
                        writer.write(tmpAttr + "\"");
                    }else{
                        writer.write(attrValue + "\"");
                    }

                }else if( attrQName.equals("value") &&
                          currentTag.equals("parameter") &&
                          currentParamName.equals("className") ){

                    writer.write("org.glite.ce.monitor.CEMonitorService\"");

                }else if( attrQName.equals("value") &&
                          currentTag.equals("parameter") &&
                          currentParamName.equals("wsdlPortType") ){

                    writer.write("CEMonitor\"");

                }else{

                    writer.write(attrValue + "\"");

                }

            }
        }catch(IOException ioEx){
            throw new SAXParseException(ioEx.getMessage(), null);
        }

        tagIncomplete = true;
        indent++;

    }

    public void endElement(String uri, String name, String qName)
        throws SAXException {

        indent--;

        try{
            if( tagIncomplete ){
                writer.write("/>\n");
            }else{
                if( currentToken==null )
                    indent();
                writer.write("</" + qName + ">\n");
            }

            tagIncomplete = false;
            currentToken = null;
            writer.flush();
        }catch(IOException ioEx){
            throw new SAXParseException(ioEx.getMessage(), null);
        }
    }

    public void characters(char[] ch, int start, int length)
        throws SAXException {

        currentToken = (new String(ch, start, length)).trim();
        if( currentToken.length()==0 ){

            currentToken = null;

        }else{

            try{
                if( tagIncomplete )
                    writer.write(">");
                writer.write(currentToken);
            }catch(IOException ioEx){
                throw new SAXParseException(ioEx.getMessage(), null);
            }

            tagIncomplete = false;

        }
    }

    private void indent() throws IOException{
        for(int r=0; r<indent; r++) writer.write(" ");
    }



    public static void main(String[] args){

        StringReader reader = null;
        BufferedWriter writer = null;

        try{

            WSDDFileConfiguration wsddConf = new WSDDFileConfiguration("server-config.wsdd");

            Admin admin = new Admin();
            AxisEngine engine = new AxisServer(wsddConf);
            Document doc = XMLUtils.newDocument(new FileInputStream(args[0]));

            WSDDDocument wsddDoc = new WSDDDocument(doc);
            WSDDEngineConfiguration config = (WSDDEngineConfiguration)engine.getConfig();
            WSDDDeployment deployment = config.getDeployment();
            wsddDoc.deploy(deployment);
            engine.refreshGlobalOptions();
            engine.saveConfiguration();

            writer = new BufferedWriter(new FileWriter(args[1]));
            ServerConfigParser parser = new ServerConfigParser(writer, "CEMonitor", "1.6.0");

            writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");

            SAXParserFactory spf=SAXParserFactory.newInstance();
            spf.setValidating(false);
            SAXParser sp = spf.newSAXParser();

            reader = new StringReader(wsddConf.toString());
            InputSource input=new InputSource(reader);
            input.setSystemId("file://" + new File(args[0]).getAbsolutePath());

            sp.parse(input,parser);
            writer.write("\n");

        }catch(Exception ex){
            ex.printStackTrace();
        }finally{
            if( writer!=null )
                try{ writer.close(); }catch(Exception ex){}
            if( reader!=null )
                try{ reader.close(); }catch(Exception ex){}
        }
        
    }

}
